/***************************************************************
 * Name:      wxextthread.h
 * Purpose:   Extended thread class.
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2011-02-08
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXEXTTHREAD_H
#define WXEXTTHREAD_H

//////////////////////////////////////////////////////////
//  class wxExtThread
//
/// \brief Extended thread class.
///
/// wxExtThread extends the wxThread class by some methods that send messages
/// to the owning wxEventHandler object.
//
class wxExtThread : public wxThread
{
  public:
    /// \brief Standard constructor
    ///
    /// \param poEvtHandler             A pointer to the event handler that owns the thread.
    /// \param nEventInt                A possible ID to post to the event handler.
    wxExtThread( wxEvtHandler *poEvtHandler, int nEventInt= 0 )
      : wxThread()
    {
        m_poEvtHandler= poEvtHandler;
        m_nEventInt= nEventInt;
    }

    /// Virtual destructor.
    virtual ~wxExtThread()
    {
    }

    /// \brief Thread execution starts here.
    virtual ExitCode Entry()
    {
        // Send the entry event.
        AddPendingEvent( ID_ENTRY );

        // Return with exit code 0 (success).
        return 0;
    }

    /// \brief Thread termination.
    ///
    /// Note: This handler function ist called when the thread exits whether it
    ///       terminates normally or is stopped by as call to Delete() (but NOT
    ///       when it is KILLed!).
    virtual void OnExit()
    {
        // Send the exit event.
        AddPendingEvent( ID_EXIT );
    }

    /// \brief Helper function to send events to the owning event handler.
    ///
    /// \param nEventId                 multipurpos inter (e. g. as an identifier for the
    ///                                 thread)
    virtual void AddPendingEvent( const long nEventId )
    {
        wxCommandEvent oEvent( wxEVT_COMMAND_MENU_SELECTED, nEventId );

        oEvent.SetInt( m_nEventInt );
        m_poEvtHandler->AddPendingEvent( oEvent );
    }

  protected:
    // Member varables
    wxEvtHandler *m_poEvtHandler;   ///< A pointer to the event handler that owns this thread.
    int m_nEventInt;                ///< An event value (e. g. for identifying the thread).

  public:
    // Command IDs
    enum
    {
        ID_ENTRY= 65312,
        ID_EXIT
    };
};

#endif // WXEXTTHREAD_H
