/***************************************************************
 * Name:      wxiscanglobals.h
 * Purpose:   Definition of some global functions for image
 *            manipulation etc.
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2010-11-21
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXISCANGLOBALS_H_INCLUDED
#define WXISCANGLOBALS_H_INCLUDED

/////////////////////////////////////////////////////////////////////////////
//  Global image manipulation functions
//
/// \brief Convert an image to monochrome (black and white).
///
/// \param oSrcImage        the image to transformed
/// \param nMonoThreshold   value for deciding if a point is black or white
///
/// NOTE:
///       The returned object is the transformed image. The original
///       image is left intact.
///
wxImage Image2Mono( const wxImage &oSrcImage, int nMonoThreshold= 128 );

/// \brief Convert an image to greyscale.
///
/// \param oSrcImage        the image to transformed
///
/// NOTE:
///       The returned object is the transformed image. The original
///       image is left intact.
///
inline wxImage Image2Grey( const wxImage &oSrcImage )
{
    return oSrcImage.ConvertToGreyscale();
}

/// \brief Crop an image to the given size.
///
/// \param oSrcImage        the image to transformed
/// \param x                x-coordinate in pixels of the upper left corner of the cropping region.
/// \param y                y-coordinate in pixels of the upper left corner of the cropping region.
/// \param w                Width of the cropping region in pixels.
/// \param h                Height of the cropping region in pixels.
///
/// NOTE:
///       The returned object is the transformed image.
///
inline wxImage CropImage( wxImage oImage, int x, int y, int w, int h )
{
    oImage.Resize( wxSize( w + x, h + y ), wxPoint( 0, 0 ), 255, 255, 255 );
    return oImage.GetSubImage( wxRect( x, y, w, h ) );
}

/// \brief Modify brightness, contrast, gamma value, red, green
///        and blue channel of an image.
///
/// \param oSrcImage        the image to be transformed
/// \param nBrightness      adjustment value for brightness
/// \param nContrast        adjustment value for constrast
/// \param nGamma           adjustment value for gamma value * 100
/// \param nRed             adjustment value for red value
/// \param nGreen           adjustment value for green value
/// \param nBlue            adjustment value for blue value
///
/// NOTE:
///       The returned object is the transformed image. The original
///       image is left intact.
///
wxImage EnhenceColours( const wxImage &oSrcImage,
                        int nBrightness= 0, int nContrast= 0, int nGamma= 100,
                        int nRed= 0, int nGreen= 0, int nBlue= 0 );

/// \brief Fine rotate an image by its centre.
///
/// \param oSrcImage        the image to be transformed
/// \param nAngleRadian     the angle in radian
///
/// NOTE:
///       The returned object is the transformed image. The original
///       image is left intact.
///
inline wxImage RotateImage( const wxImage &oSrcImage, double nAngleRadian )
{
    return oSrcImage.Rotate( nAngleRadian, wxPoint( oSrcImage.GetWidth() / 2, oSrcImage.GetHeight() / 2 ), true, NULL );
}

/// \brief Resize an image to the given size.
///
/// \param oSrcImage        the image to be transformed
/// \param nDestWidth       the desired width of the destination image
/// \param nDestHeight      the desired height of the destination image
///
/// NOTE:
///       The returned object is the transformed image. The original
///       image is left intact.
///
inline wxImage ResizeImage( const wxImage &oSrcImage, int nDestWidth, int nDestHeight )
{
    return oSrcImage.Scale( nDestWidth, nDestHeight, wxIMAGE_QUALITY_HIGH );
}

/////////////////////////////////////////////////////////////////////////////
//  Global clipboard functions
//
/// \brief Copy the given text to the sytem clipboard.
///
/// \param strText          the text to copy to the clipboard
///
bool CopyText2Clipboard( const wxString &strText );

/// \brief Copy the given image to the sytem clipboard.
///
/// \param oImage           the image to copy to the clipboard
///
bool CopyImage2Clipboard( const wxImage &oImage );

/////////////////////////////////////////////////////////////////////////////
//  Global multiple usage functions
//
/// \brief Load the content of a text file into memory.
///
/// \param strFilename      the name of the text file
///
wxString GetTextFromFile( const wxString &strFilename );

#endif // WXISCANGLOBALS_H_INCLUDED
